/**
 * @file
 * Decrypts cipher texts into a file given a correct privkey passphrase.
 */
(function($, Drupal, once) {
  const ProcBehavior = {
    async attach(context, settings) {
      // If context is the decryption form or this is standalone mode:
      if (context.id || drupalSettings.proc.proc_decryption_mode === '0') {
        const drupalSettingsProc = drupalSettings.proc;
        const messages = new Drupal.Message();
        const decryptionLink = $('#decryption-link');
        const basePath = drupalSettingsProc.base_path;
        const _navigator = {};
        for (const i in navigator) _navigator[i] = navigator[i];
        const passowrdSeesionCachingSalt = JSON.stringify(_navigator);

        async function decrypt() {
          const cache_password = document.querySelector(
            '[name="cache_password"]',
          )
            ? document.querySelector('[name="cache_password"]').value
            : 0;
          const secretPassString = $('input[name=password]')[0].value;
          const passphrase = drupalSettingsProc.proc_pass.concat(
            secretPassString,
          );
          const privateKey = await openpgp.readPrivateKey({
            armoredKey: drupalSettingsProc.proc_privkey,
          });
          const keyringCacheType = drupalSettingsProc.proc_keyring_type;

          if (cache_password === '1' && keyringCacheType !== 'keyring') {
            // Cache password.
            await (async () => {
              const message = await openpgp.createMessage({
                text: secretPassString,
              });
              const encrypted = await openpgp.encrypt({
                message,
                passwords: [passowrdSeesionCachingSalt],
              });
              // Check if an expiration time is already set.
              if (
                sessionStorage.getItem(
                  `proc.password_cache_type.key_user_id.${drupalSettings.user.uid}`,
                )
              ) {
                const remainingTime =
                  sessionStorage.getItem(
                    `proc.password_cache_type.key_user_id.${drupalSettings.user.uid}`,
                  ) - new Date().getTime();
                if (remainingTime <= 0) {
                  // Remove cached password.
                  sessionStorage.removeItem(
                    `proc.password_cache.key_user_id.${drupalSettings.user.uid}`,
                  );
                  sessionStorage.removeItem(
                    `proc.password_cache_type.key_user_id.${drupalSettings.user.uid}`,
                  );
                }
              } else {
                sessionStorage.setItem(
                  `proc.password_cache.key_user_id.${drupalSettings.user.uid}`,
                  encrypted,
                );
                // If expiration time is set to 1 hour, then count 1 hour from now.
                let expirationTime = new Date().getTime() + 3600000;
                // If expiration time is set to 2 hours, then count 2 hours from now.
                if (keyringCacheType === 'keyring_2h') {
                  expirationTime += 3600000;
                }
                sessionStorage.setItem(
                  `proc.password_cache_type.key_user_id.${drupalSettings.user.uid}`,
                  expirationTime,
                );
              }
            })();
          }
          if (cache_password === '0' || keyringCacheType === 'keyring') {
            // Remove password from cache.
            sessionStorage.removeItem(
              `proc.password_cache.key_user_id.${drupalSettings.user.uid}`,
            );
            // Remove expiration time from cache.
            sessionStorage.removeItem(
              `proc.password_cache_type.key_user_id.${drupalSettings.user.uid}`,
            );
            $('input[name=password]')[0].value = '';
          }

          if (!$('#proc-decrypting-info')[0]) {
            $('form#-proc-decrypt-to-file').prepend(
              `<div class="messages messages--info proc--info" id="proc-decrypting-info">${drupalSettingsProc.proc_labels.proc_introducing_decryption}</div>`,
            );
          }

          const temporaryDownloadLink = document.createElement('a');
          temporaryDownloadLink.style.display = 'none';
          document.body.appendChild(temporaryDownloadLink);

          await Promise.all(
            drupalSettingsProc.proc_ids.map(async (cipherId, cipherIndex) => {
              const cache = 'caches' in self ? caches.open('proc') : null;
              let response;
              let cipher;
              let cachedCiphers;

              if (cache) {
                cachedCiphers = await cache.then(async function(cache) {
                  response = await cache.match(procURLs[cipherIndex]);
                  if (!response) {
                    console.info('Adding cipher to cache');
                    const cipherResponse = await fetch(procURLs[cipherIndex]);
                    cipher = cipherResponse.clone();
                    await cache.put(procURLs[cipherIndex], cipherResponse);
                    return cipher;
                    // @TODO: add clean up of expired cache if any!
                  }
                  console.info('Reusing cipher from cache');
                  return response;
                });
              }

              const cipherText = cachedCiphers
                ? (await cachedCiphers.json()).pubkey[0].armored
                : '';
              const decryptedPrivateKey = await openpgp
                .decryptKey({
                  privateKey,
                  passphrase,
                })
                .catch(function(err) {
                  const decryptionLinkElement = $('a#decryption-link');
                  $('form#-proc-decrypt-to-file').prepend(
                    `<div class="messages error">${Drupal.t(err)}</div>`,
                  );
                  if (decryptionLinkElement[0].href) {
                    const fileUrl = decryptionLinkElement[0].href;
                    URL.revokeObjectURL(fileUrl);
                    decryptionLinkElement.removeAttr('href');
                  }
                });

              if (drupalSettingsProc.proc_signed[cipherIndex] !== 0) {
                const expectedSignedValue = true;
                response = await fetch(
                  `${window.location.origin +
                    Drupal.settings.basePath}proc/api/getpubkey/${
                    drupalSettingsProc.proc_signed[cipherIndex]
                  }/pid`,
                );
                const pubkeysJson = await response.json();
                const pubKeyObj = (
                  await openpgp.key.readArmored(pubkeysJson.pubkey[0].key)
                ).keys[0];
                const optionsVerify = {
                  message: await openpgp.message
                    .readArmored(cipherText)
                    .catch(function(err) {
                      $('form#-proc-decrypt-to-file').prepend(
                        `<div class="messages error">${Drupal.t(err)}</div>`,
                      );
                    }),
                  publicKeys: pubKeyObj,
                  compression: openpgp.enums.compression.zip,
                };
                const decrypted = await openpgp.verify(optionsVerify);
                decrypted.signatures[0].verified.then(val => {
                  if (val) {
                    console.info(
                      `signed by key id ${decrypted.signatures[0].keyid.toHex()}`,
                    );
                  } else {
                    $('form#-proc-sign-file').prepend(
                      `<div class="messages error">${Drupal.t(
                        'Signature could not be verified',
                      )}</div>`,
                    );
                    throw new Error('Signature could not be verified');
                  }
                });
                if (decrypted) {
                  await openFileAction(
                    decrypted,
                    temporaryDownloadLink,
                    cipherIndex,
                  );
                }
              } else {
                const message = await openpgp.readMessage({
                  armoredMessage: cipherText,
                });
                let procFormat = 'binary';
                if (drupalSettingsProc.proc_sources_input_modes[cipherIndex]) {
                  procFormat =
                    drupalSettingsProc.proc_sources_input_modes[cipherIndex];
                }
                const decrypted = await openpgp
                  .decrypt({
                    decryptionKeys: decryptedPrivateKey,
                    message,
                    format: procFormat,
                  })
                  .catch(function(err) {
                    $('.messages').after(
                      `<div class="messages messages--error proc--error" id="proc-decrypting-status">${Drupal.t(
                        err,
                      )}</div>`,
                    );
                  });
                if (decrypted === undefined) {
                  const msg = Drupal.t(
                    'Unable to decrypt the content. Make sure you have entered the right passphrase.',
                  );
                  // Log warning on console.
                  console.warn(msg);
                  // Prepare feedback message for the user.
                  const DecryptErrorAjaxMsg = Drupal.ajax({
                    url: `/proc/modal-message?msg=${msg}`,
                    dialogType: 'modal',
                    dialog: { width: 400 },
                  });
                  // Display the error on a modal.
                  DecryptErrorAjaxMsg.execute();
                }
                if (decrypted) {
                  if (
                    $('textarea').length !== 0 &&
                    (typeof decrypted.data === 'string' ||
                      decrypted.data instanceof String)
                  ) {
                    $(`#edit-${cipherId}`)[0].innerText = decrypted.data;
                  } else {
                    await openFileAction(
                      decrypted,
                      temporaryDownloadLink,
                      cipherIndex,
                    );
                  }
                }
              }
            }),
          );
          document.body.removeChild(temporaryDownloadLink);
        }

        // Initialize password caching.
        if (document.querySelector('[name="cache_password"]')) {
          // Caching password is disabled by default.
          document.querySelector('[name="cache_password"]').value = 0;
          if (drupalSettingsProc.proc_cache_password_mode === '2') {
            // Caching password is enabled by default.
            document.querySelector('[name="cache_password"]').value = 1;
          }
        }
        if (
          sessionStorage.getItem(
            `proc.password_cache.key_user_id.${drupalSettings.user.uid}`,
          )
        ) {
          try {
            await (async () => {
              const messageSource = await openpgp.readMessage({
                armoredMessage: sessionStorage.getItem(
                  `proc.password_cache.key_user_id.${drupalSettings.user.uid}`,
                ),
              });
              const { data: decrypted } = await openpgp.decrypt({
                message: messageSource,
                passwords: [passowrdSeesionCachingSalt],
              });
              $('input[name=password]')[0].value = decrypted;
              // Do not decrypt automatically in standalone mode.
              if (drupalSettings.proc.proc_decryption_mode !== '0') {
                decrypt();
              }
            })();
          } catch (err) {
            // This error may happen because of a change in the browser fingerprint.
            console.info(err);
          }
        }
        if (!window.Blob) {
          messages.add(drupalSettingsProc.proc_labels.proc_fileapi_err_msg, {
            type: 'error',
          });
          return;
        }

        const procURLs = drupalSettingsProc.proc_ids.map(
          (cipherId, cipherIdIndex) =>
            `${window.location.origin +
              basePath}api/proc/getcipher/${cipherId}/?cipherchanged=${
              drupalSettingsProc.procs_changed[cipherIdIndex]
            }`,
        );

        const openFileAction = async (
          decrypted,
          temporaryDownloadLink,
          cipherIndex,
        ) => {
          const plaintext = decrypted.data;
          const blob = new Blob([plaintext], {
            type: 'application/octet-binary',
            endings: 'native',
          });
          const link = decryptionLink;

          temporaryDownloadLink.setAttribute('href', URL.createObjectURL(blob));
          const openActionLabel =
            drupalSettingsProc.proc_labels.proc_open_file_state;

          if (link.text() !== openActionLabel) {
            link.text(openActionLabel);
            link.removeClass('active');
            $('.messages').after(
              `<div class="messages status" id="proc-decrypting-status">${drupalSettingsProc.proc_labels.proc_decryption_success}</div>`,
            );
          }

          if (
            blob.size.toString() ===
              drupalSettingsProc.proc_sources_file_sizes[cipherIndex] ||
            drupalSettingsProc.proc_skip_size_mismatch === 'TRUE'
          ) {
            temporaryDownloadLink.setAttribute(
              'download',
              drupalSettingsProc.proc_sources_file_names[cipherIndex],
            );

            if (drupalSettingsProc.proc_decryption_mode === '0') {
              temporaryDownloadLink.click();
            }

            drupalSettings.proc.proc_field_name = drupalSettings.proc.proc_field_name.replace(
              /'/g,
              '',
            );

            if (drupalSettingsProc.proc_decryption_mode === '1') {
              const dialog = document.querySelector(
                `.class-${drupalSettings.proc.proc_field_name}`,
              );

              if (drupalSettings.proc.proc_in_mode === '2') {
                blob.text().then(val => {
                  if (val) {
                    $(
                      `[name^=${drupalSettings.proc.proc_field_name}]`,
                    )[0].value = val;
                    $(
                      `[name^=${drupalSettings.proc.proc_field_name}]`,
                    )[0].disabled = false;

                    dialog.parentNode.removeChild(dialog);

                    if (
                      drupalSettingsProc.proc_decryption_trigger_fields === '1'
                    ) {
                      const slaveFields = document.querySelectorAll(
                        '[id^="encrypt-checkbox"]',
                      );
                      for (let i = 0; i < slaveFields.length; i++) {
                        if (
                          slaveFields[i].id !=
                            `encrypt-checkbox-${drupalSettings.proc.proc_field_name}` &&
                          slaveFields[i].checked
                        ) {
                          document
                            .querySelector(`#${slaveFields[i].id}`)
                            .click();
                          break;
                        }
                      }
                    }
                  }
                });
              }

              if (drupalSettings.proc.proc_in_mode === '0') {
                temporaryDownloadLink.click();
                dialog.parentNode.removeChild(dialog);
              }
            }
          } else {
            $('form#-proc-decrypt-to-file').prepend(
              `<div class="messages error">${drupalSettingsProc.proc_labels.proc_decryption_size_mismatch}</div>`,
            );
          }
        };

        $(once('on', 'input#edit-pass', context)).on('focusin', function() {
          messages.clear();
          this.value = '';
          const actionLink = decryptionLink;
          if (!actionLink.hasClass('active')) {
            actionLink
              .text('Decrypt')
              .removeClass('active')
              .removeAttr('download')
              .removeAttr('href');
          }
        });

        $('#proc-decrypt-form').submit(function(e) {
          e.preventDefault();
          decryptionLink.click();
        });

        decryptionLink.off().on('click', async function(e) {
          e.preventDefault();
          await decrypt();
        });
      }
    },
  };
  Drupal.behaviors.ProcBehavior = ProcBehavior;
})(jQuery, Drupal, once);
