<?php

namespace Drupal\proc;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityChangedInterface;
use Drupal\user\EntityOwnerInterface;

/**
 * Provides an interface defining a Proc entity.
 *
 * We have this interface so we can join the other interfaces it extends.
 *
 * @ingroup proc
 */
interface ProcInterface extends ContentEntityInterface, EntityOwnerInterface, EntityChangedInterface {

  /**
   * Content encryption strategies.
   *
   * Restrictive encryption strategy requires the existance of valid PGP key for
   * each and every recipient of a protected content.
   * When using the restritive encryption strategy, protected content can only
   * be created when all recipients have a valid PGP key.
   *
   * Permissive encryption strategy filter-out from encryption recipients all
   * users without valid PGP key.
   * When using the permissive encryption strategy, protected content is created
   * skipping any user from the list of recipients that still has no PGP key.
   */

  const STRATEGY_RESTRICTIVE = 0;
  const STRATEGY_PERMISSIVE = 1;

  /**
   * Proc field operations.
   *
   * Disabled:
   * Only encryption:
   * Encryption and Signature:
   */
  const FIELD_OPS_DISABLED = 0;
  const FIELD_OPS_ONLY_ENCRYPTION = 1;
  const FIELD_OPS_ENCRYPTION_AND_SIGNATURE = 2;

  /**
   * Proc field input modes.
   *
   * File: A file field for encprytion of documents.
   * Textarea: A textarea field for multi line strings encryption.
   * Textfield: A text field for encryption of strings.
   * Datefield: A datetime field for encryption of timestamp.
   */
  const INPUT_MODE_FILE = 0;
  const INPUT_MODE_TEXTAREA = 1;
  const INPUT_MODE_TEXTFIELD = 2;
  const INPUT_MODE_DATE = 3;

  /**
   * Decryption mode.
   *
   * New page: Opens the decrypt form in a new page.
   * Dialog: Opens the decrypt form in a dialog box.
   */
  const DECRYPT_MODE_NEW_PAGE = 0;
  const DECRYPT_MODE_DIALOG = 1;

  /**
   * Password caching options.
   *
   * Disabled: No password caching.
   * Allow: Provides a checkbox option for users define password caching.
   * Apply: Hides the checkbox option and apply password caching by default.
   */
  const PASS_CACHE_OPS_DISABLED = 0;
  const PASS_CACHE_OPS_ALLOW = 1;
  const PASS_CACHE_OPS_APPLY = 2;

  /**
   * Proc encryption trigger format.
   *
   * Buttons: Show encrypt and decrypt buttons.
   * Checkbox: Use a checkbox switcher for encrypt and decrypt.
   */
  const TRIGGER_FORMAT_BUTTONS = 0;
  const TRIGGER_FORMAT_CHECKBOX = 1;

  /**
   * Fetcher filter types.
   *
   * Id: A list of user ids in a JSON format.
   *  e.g.: [{'id': 1}, {'id': 10}, {'id': 12}]
   * Name: A list of usernames in a JSON format.
   *  e.g. [{name: 'foo'}, {'name': 'bar'}, {'name': 'baz'}]
   */
  const FETCHER_FILTER_TYPE_ID = 0;
  const FETCHER_FILTER_TYPE_NAME = 1;

}
